package cn.wustlinghang.wusthelper.internal.graduate.services;

import cn.wustlinghang.mywust.core.request.service.auth.GraduateLogin;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.mywust.network.RequestClientOption;
import cn.wustlinghang.wusthelper.internal.rpc.exception.GraduateRpcException;
import jakarta.enterprise.context.ApplicationScoped;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.microprofile.config.inject.ConfigProperty;

import java.io.IOException;

@Slf4j
@ApplicationScoped
public class LoginService {

    private final RequestClientOption requestClientOption;
    private final GraduateLogin graduateLogin;
    private final int maxRetryTimes;

    public LoginService(RequestClientOption requestClientOption,
                        GraduateLogin graduateLogin,
                        @ConfigProperty(name = "graduate.max-retry-times", defaultValue = "4")
                        int maxRetryTimes) {

        this.requestClientOption = requestClientOption;
        this.graduateLogin = graduateLogin;

        this.maxRetryTimes = maxRetryTimes;
    }

    public String login(String username, String password) throws GraduateRpcException {
        try {
            return graduateLogin.getLoginCookie(username, password, maxRetryTimes, requestClientOption);
        } catch (ApiException e) {
            throw wrapApiException(e);
        } catch (IOException e) {
            throw new GraduateRpcException(
                    GraduateRpcException.TypeCode.NETWORK_EXCEPTION,
                    GraduateRpcException.SubModuleCode.AUTH,
                    GraduateRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    public boolean verify(String cookie) throws GraduateRpcException {
        try {
            graduateLogin.checkCookies(cookie, requestClientOption);
            return true;
        } catch (IOException e) {
            throw new GraduateRpcException(
                    GraduateRpcException.TypeCode.NETWORK_EXCEPTION,
                    GraduateRpcException.SubModuleCode.AUTH,
                    GraduateRpcException.ErrorCode.NETWORK_ERROR
            );
        } catch (ApiException e) {
            if (e.getCode() == ApiException.Code.NETWORK_EXCEPTION) {
                throw new GraduateRpcException(
                        GraduateRpcException.TypeCode.NETWORK_EXCEPTION,
                        GraduateRpcException.SubModuleCode.AUTH,
                        GraduateRpcException.ErrorCode.NETWORK_ERROR
                );
            }

            return false;
        }
    }

    private GraduateRpcException wrapApiException(ApiException e) {
        GraduateRpcException.ErrorCode errorCode = switch (e.getCode()) {
            case NETWORK_EXCEPTION -> GraduateRpcException.ErrorCode.NETWORK_ERROR;
            case GRADUATE_PASSWORD_WRONG -> GraduateRpcException.ErrorCode.AUTH_PASSWORD_WRONG;
            case GRADUATE_CAPTCHA_WRONG -> GraduateRpcException.ErrorCode.AUTH_CAPTCHA_WRONG;
            default -> {
                log.error("研究生：登录代理请求异常，异常未处理");
                log.error("异常：", e);
                yield GraduateRpcException.ErrorCode.AUTH_UNKNOWN_ERROR;
            }
        };

        return new GraduateRpcException(
                GraduateRpcException.TypeCode.AUTH_EXCEPTION,
                GraduateRpcException.SubModuleCode.AUTH,
                errorCode
        );
    }
}
