package cn.wustlinghang.wusthelper.internal.undergrad.rpc;

import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.RandomUtil;
import cn.wustlinghang.mywust.network.Requester;
import cn.wustlinghang.wusthelper.internal.undergrad.rpc.client.ConsulClient;
import cn.wustlinghang.wusthelper.internal.undergrad.rpc.client.FrpcClient;
import cn.wustlinghang.wusthelper.internal.undergrad.rpc.config.RpcConfig;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.quarkus.runtime.Startup;
import jakarta.annotation.PostConstruct;
import jakarta.annotation.PreDestroy;
import jakarta.enterprise.context.ApplicationScoped;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.microprofile.config.inject.ConfigProperty;

@Slf4j
@Startup
@ApplicationScoped
public class FrpConsulRegister {

    @ConfigProperty(name = "quarkus.http.port")
    String localServicePort;

    private final String serviceId;

    private final RpcConfig rpcConfig;

    private final FrpcClient frpcClientClient;

    private final ConsulClient consulClient;

    private boolean registered = false;

    public FrpConsulRegister(RpcConfig rpcConfig, FrpcClient frpcClientClient, Requester requester,
                             ObjectMapper objectMapper) {
        this.rpcConfig = rpcConfig;

        this.frpcClientClient = frpcClientClient;

        this.serviceId = RandomUtil.randomString(8);

        this.consulClient = new ConsulClient(
                rpcConfig.getServiceName(), this.serviceId,
                rpcConfig.getConsulAddress(),
                objectMapper, requester);
    }

    @PostConstruct
    public void onStartup() {
        ThreadUtil.execute(() -> {
            try {
                log.info("连接frp并注册consul...");
                this.doRegister();
                log.info("注册完毕");
            } catch (Exception e) {
                log.warn("注册服务时发生异常：", e);
                log.warn("不进行转发注册，直接服务");
            }
        });

        Runtime.getRuntime().addShutdownHook(new Thread(this::onShutdown, "ShutdownHookThread"));
    }

    @PreDestroy
    public void onShutdown() {
        try {
            if (this.registered) {
                log.info("注销服务...");
                this.consulClient.deregister();
                this.frpcClientClient.removeProxy();
                log.info("服务注销完毕");
                this.registered = false;
            }
        } catch (Exception e) {
            log.warn("注销服务时发生异常：", e);
        }
    }

    private void doRegister() throws Exception {
        frpcClientClient.addFrpProxy(
                rpcConfig.getServiceName(), serviceId,
                "127.0.0.1", localServicePort
        );

        String remoteAddress;
        String[] remote;
        int retry = 0;
        do {
            // 先睡个0.5秒等待连接成功再获取状态读端口
            ThreadUtil.sleep(500);

            JsonNode proxy = frpcClientClient.getProxyStatus(rpcConfig.getServiceName(), serviceId);
            remoteAddress = proxy.path("remote_addr")
                    .asText("127.0.0.1:" + localServicePort);
            remote = remoteAddress.split(":");
            retry++;
        } while (retry < 3 && remote.length < 2);
        if (retry == 3) {
            throw new Exception("获取frp隧道信息重试次数过多，请手动添加隧道和注册中心");
        }

        this.consulClient.register(remoteAddress, Integer.parseInt(remote[1]));
        this.registered = true;
    }
}
